classdef CameraConfig < handle
  % CameraConfig
  %   This class allows you to view/edit the camera settings
  %
  % Methods:
  %   write - Write camera config to disk
  %   getType - Get the camera type
  %   reset - Reset camera config to default
  %   frameSize - Get the amount of frames in the config. Having multiple frames
  %               allows you to configure different modulation frequencies or
  %               integration times per frame.
  %   addFrame - Add a default frame to the camera config
  %   defaultProcessing - Generate default processing config from camera
  %                       configuration
  %   eraseFrame - Erase specified frame
  %   getPhaseShifts - Get the phase shifts. Value ranges from 0.0 to 1.0 which
  %                    translates to 0-2 pi
  %   setPhaseShifts - Set the phase shifts. Value ranges from 0.0 to 1.0 which
  %                    translates to 0-2 pi
  %   getIntegrationTime - Get the integration time
  %   setIntegrationTime - Set the integration time
  %   getSensorMode - Get the mode to set the camera's depth sensor to
  %   setSensorMode - Set the mode to set the camera's depth sensor to
  %   getModulationFrequency - Get the modulation frequency
  %   setModulationFrequency - Set the modulation frequency
  %   getDutyCycle - Get the duty cycle
  %   setDutyCycle - Set the duty cycle
  %   getRoi - Get the region of interest
  %   setRoi - Set the region of interest
  %   getBinning - Get the amount of binning
  %   setBinning - Set the amount of binning
  %   getFlip - Get is the image flipped
  %   setFlip - Set is the image flipped
  %   getMirror - Get is the image mirrored
  %   setMirror - Set is the image mirrored
  %   getGain - Get the image sensor gain
  %   setGain - Set the image sensor gain
  %   getSyncMode - Get the kind of camera synchronization
  %   setSyncMode - Set the kind of camera synchronization
  %   maximumIntegrationTime - Get the maximum integration time possible
  %   getDac - Get the DAC gain
  %   setDac - Set the DAC gain
  %   getMlxMipiSpeed - Get the mipi speed for the MLX75027
  %   setMlxMipiSpeed - Set the mipi speed for the MLX75027
  %   getMipiSpeed - Get the mipi speed
  %   setMipiSpeed - Set the mipi speed
  %   getMlxPreheat - Get the preheat time
  %   setMlxPreheat - Set the preheat time
  %   getMlxPreheatEnable - Get enable preheat
  %   setMlxPreheatEnable - Set enable preheat
  %   setOptKeaNiosHex - Set the Nios hex for the keaB cameras
  %   getOptKeaSubQuad - Get the opt8241 number of sub and quad frames
  %   setOptKeaSubQuad - Set the opt8241 number of sub and quad frames
  %   getProcessing - Get the processing configuration for when the depth is
  %                   calculated on camera
  %   setProcessing - Set the processing configuration for when the depth is
  %                   calculated on camera
  %   getFrameTime - Get the time this depth frame will take. This is useful
  %                  when you want a steady but lower framerate.
  %   setFrameTime - Set the time this depth frame will take. This is useful
  %                  when you want a steady but lower framerate.
  %   getGlobalPhaseOffset - Get phase offset for illumination and sensor. The
  %                          phase offsets are two floating point values
  %                          corresponding to the phase delay for the
  %                          illumination and sensor respectively. The values
  %                          should be between 0.0 and 1.0 where 0.0 is no phase
  %                          offset and 1.0 is 360 degree phase offset
  %   setGlobalPhaseOffset - Set phase offset for illumination and sensor. The
  %                          phase offsets are two floating point values
  %                          corresponding to the phase delay for the
  %                          illumination and sensor respectively. The values
  %                          should be between 0.0 and 1.0 where 0.0 is no phase
  %                          offset and 1.0 is 360 degree phase offset
  %   getIlluminationMode - Get the illumination mode
  %   setIlluminationMode - Set the illumination mode
  %   getImageOffsets - Get the image offsets. This configuration is only for
  %                     sensors with column phase offsets.
  %   setImageOffsets - Set the image offsets. This configuration is only for
  %                     sensors with column phase offsets.
  %   getRgbWidth - Get the width of the rgb sensor
  %   getRgbHeight - Get the height of the rgb sensor
  %   getTriggerMode - Get the trigger mode
  %   setTriggerMode - Set the trigger mode
  %   getDepthSensorType - Get the depth sensor type
  %   getSensorBoardType - Get the sensor board type
  %   getLightSourceType - Get the light source type
  %   getRgbSensorType - Get the rgb sensor type
  %   getDepthFps - Get the maximum theoretical fps the camera can run
  %   getName - Get the configuration name
  %   setName - Set the configuration name
  %   getDescription - Get the description
  %   setDescription - Set the description
  %   getLightPower - Get the light power
  %   setLightPower - Set the light power
  %   hasLightPower - Light power is set
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = CameraConfig(varargin)
      % CameraConfig
      %   Construct new camera_config class
      %
      % ----------
      %
      %   Read camera config from disk
      %
      % Inputs:
      %   1 - string - Location of camera config on disk
      %
      % ----------
      %
      %   Create camera config from camera type
      %
      % Inputs:
      %   1 - CameraType - Camera type
      %
      % ----------
      %
      %   Create camera config from the four types
      %
      % Inputs:
      %   1 - DepthSensorType - Depth sensor type
      %   2 - SensorBoardType - Sensor board type
      %   3 - LightSourceType - Light source type
      %   4 - RgbSensorType - RGB sensor type
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      elseif nargin == 1 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        obj.mPtr = tof.mex('tof_camera_config_new_from_disk', varargin{1});
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_camera_config_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function write(obj, file_location)
      % write
      %   Write camera config to disk
      %
      % Inputs:
      %   file_location - string - Location to save camera config to
      %

      tof.mex('tof_camera_config_write', obj.mPtr, file_location);
    end

    function ret_val = getType(obj)
      % getType
      %   Get the camera type
      %
      % Output:
      %   ret_val - CameraType - Camera type
      %

      ret_val = tof.mex('tof_camera_config_get_type', obj.mPtr);
    end

    function reset(obj)
      % reset
      %   Reset camera config to default
      %

      tof.mex('tof_camera_config_reset', obj.mPtr);
    end

    function ret_val = frameSize(obj)
      % frameSize
      %   Get the amount of frames in the config. Having multiple frames allows
      %   you to configure different modulation frequencies or integration times
      %   per frame.
      %
      % Output:
      %   ret_val - numeric - Frame size
      %

      ret_val = tof.mex('tof_camera_config_frame_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function addFrame(obj)
      % addFrame
      %   Add a default frame to the camera config
      %

      tof.mex('tof_camera_config_add_frame', obj.mPtr);
    end

    function ret_val = defaultProcessing(obj)
      % defaultProcessing
      %   Generate default processing config from camera configuration
      %
      % Output:
      %   ret_val - ProcessingConfig - The default processing config
      %

      ret_val = tof.mex('tof_camera_config_default_processing', obj.mPtr);
      ret_val = tof.ProcessingConfig(ret_val);
    end

    function eraseFrame(obj, frame)
      % eraseFrame
      %   Erase specified frame
      %
      % Inputs:
      %   frame - numeric - Frame number
      %

      tof.mex('tof_camera_config_erase_frame', obj.mPtr, frame);
    end

    function ret_val = getPhaseShifts(obj, frame)
      % getPhaseShifts
      %   Get the phase shifts. Value ranges from 0.0 to 1.0 which translates to
      %   0-2 pi
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - vector[numeric] - Phase shifts
      %

      ret_val = tof.mex('tof_camera_config_get_phase_shifts', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setPhaseShifts(obj, frame, phase_shifts)
      % setPhaseShifts
      %   Set the phase shifts. Value ranges from 0.0 to 1.0 which translates to
      %   0-2 pi
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   phase_shifts - vector[numeric] - Phase shifts
      %

      tof.mex('tof_camera_config_set_phase_shifts', obj.mPtr, frame, single(phase_shifts));
    end

    function ret_val = getIntegrationTime(obj, frame)
      % getIntegrationTime
      %   Get the integration time
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - vector[numeric] - Integration time in micro seconds
      %

      ret_val = tof.mex('tof_camera_config_get_integration_time', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setIntegrationTime(obj, frame, integration_time)
      % setIntegrationTime
      %   Set the integration time
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   integration_time - vector[numeric] - Integration time in micro seconds
      %

      tof.mex('tof_camera_config_set_integration_time', obj.mPtr, frame, uint32(integration_time));
    end

    function ret_val = getSensorMode(obj, frame)
      % getSensorMode
      %   Get the mode to set the camera's depth sensor to
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - SensorMode - Sensor mode
      %

      ret_val = tof.mex('tof_camera_config_get_sensor_mode', obj.mPtr, frame);
    end

    function setSensorMode(obj, frame, sensor_mode)
      % setSensorMode
      %   Set the mode to set the camera's depth sensor to
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   sensor_mode - SensorMode - Sensor mode
      %

      tof.mex('tof_camera_config_set_sensor_mode', obj.mPtr, frame, uint32(sensor_mode));
    end

    function ret_val = getModulationFrequency(obj, frame)
      % getModulationFrequency
      %   Get the modulation frequency
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - Modulation frequency in MHz
      %

      ret_val = tof.mex('tof_camera_config_get_modulation_frequency', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setModulationFrequency(obj, frame, modulation_frequency)
      % setModulationFrequency
      %   Set the modulation frequency
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   modulation_frequency - numeric - Modulation frequency in MHz
      %

      tof.mex('tof_camera_config_set_modulation_frequency', obj.mPtr, frame, modulation_frequency);
    end

    function ret_val = getDutyCycle(obj, frame)
      % getDutyCycle
      %   Get the duty cycle
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - Duty cycle 0.0 to 1.0
      %

      ret_val = tof.mex('tof_camera_config_get_duty_cycle', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setDutyCycle(obj, frame, duty_cycle)
      % setDutyCycle
      %   Set the duty cycle
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   duty_cycle - numeric - Duty cycle 0.0 to 1.0
      %

      tof.mex('tof_camera_config_set_duty_cycle', obj.mPtr, frame, duty_cycle);
    end

    function ret_val = getRoi(obj, frame)
      % getRoi
      %   Get the region of interest
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - Roi - Region of interest
      %

      ret_val = tof.mex('tof_camera_config_get_roi', obj.mPtr, frame);
      ret_val = tof.Roi(ret_val);
    end

    function setRoi(obj, frame, roi)
      % setRoi
      %   Set the region of interest
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   roi - Roi - Region of interest
      %

      tof.mex('tof_camera_config_set_roi', obj.mPtr, frame, roi.get_object_ptr());
    end

    function ret_val = getBinning(obj, frame)
      % getBinning
      %   Get the amount of binning
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - Amount of binning 0=none, 1=x2, 2=x4, 3=x8
      %

      ret_val = tof.mex('tof_camera_config_get_binning', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setBinning(obj, frame, binning)
      % setBinning
      %   Set the amount of binning
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   binning - numeric - Amount of binning 0=none, 1=x2, 2=x4, 3=x8
      %

      tof.mex('tof_camera_config_set_binning', obj.mPtr, frame, binning);
    end

    function ret_val = getFlip(obj, frame)
      % getFlip
      %   Get is the image flipped
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - logical - Image is flipped
      %

      ret_val = tof.mex('tof_camera_config_get_flip', obj.mPtr, frame);
    end

    function setFlip(obj, frame, flip)
      % setFlip
      %   Set is the image flipped
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   flip - logical - Image is flipped
      %

      tof.mex('tof_camera_config_set_flip', obj.mPtr, frame, flip);
    end

    function ret_val = getMirror(obj, frame)
      % getMirror
      %   Get is the image mirrored
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - logical - Image is mirrored
      %

      ret_val = tof.mex('tof_camera_config_get_mirror', obj.mPtr, frame);
    end

    function setMirror(obj, frame, mirror)
      % setMirror
      %   Set is the image mirrored
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   mirror - logical - Image is mirrored
      %

      tof.mex('tof_camera_config_set_mirror', obj.mPtr, frame, mirror);
    end

    function ret_val = getGain(obj)
      % getGain
      %   Get the image sensor gain
      %
      % Output:
      %   ret_val - numeric - Image sensor gain
      %

      ret_val = tof.mex('tof_camera_config_get_gain', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setGain(obj, gain)
      % setGain
      %   Set the image sensor gain
      %
      % Inputs:
      %   gain - numeric - Image sensor gain
      %

      tof.mex('tof_camera_config_set_gain', obj.mPtr, gain);
    end

    function ret_val = getSyncMode(obj)
      % getSyncMode
      %   Get the kind of camera synchronization
      %
      % Output:
      %   ret_val - numeric - The kind of sync mode. 0 waits for the vsync pulse
      %                       before starting the next frame. 1 optional vsync 2
      %                       is no frame syncing
      %

      ret_val = tof.mex('tof_camera_config_get_sync_mode', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setSyncMode(obj, sync_mode)
      % setSyncMode
      %   Set the kind of camera synchronization
      %
      % Inputs:
      %   sync_mode - numeric - The kind of sync mode. 0 waits for the vsync
      %                         pulse before starting the next frame. 1 optional
      %                         vsync 2 is no frame syncing
      %

      tof.mex('tof_camera_config_set_sync_mode', obj.mPtr, sync_mode);
    end

    function ret_val = maximumIntegrationTime(obj, frame)
      % maximumIntegrationTime
      %   Get the maximum integration time possible
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - vector[numeric] - Maximum integration time in micro seconds
      %

      ret_val = tof.mex('tof_camera_config_maximum_integration_time', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function ret_val = getDac(obj)
      % getDac
      %   Get the DAC gain
      %
      % Output:
      %   ret_val - vector[numeric] - The DAC gain
      %

      ret_val = tof.mex('tof_camera_config_get_dac', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setDac(obj, dac)
      % setDac
      %   Set the DAC gain
      %
      % Inputs:
      %   dac - vector[numeric] - The DAC gain
      %

      tof.mex('tof_camera_config_set_dac', obj.mPtr, uint16(dac));
    end

    function ret_val = getMlxMipiSpeed(obj)
      % getMlxMipiSpeed
      %   Get the mipi speed for the MLX75027
      %
      % Output:
      %   ret_val - numeric - The mipi speed. Can only be the values 300, 600,
      %                       704, 800 and 960
      %

      ret_val = tof.mex('tof_camera_config_get_mlx_mipi_speed', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setMlxMipiSpeed(obj, mipi_speed)
      % setMlxMipiSpeed
      %   Set the mipi speed for the MLX75027
      %
      % Inputs:
      %   mipi_speed - numeric - The mipi speed. Can only be the values 300,
      %                          600, 704, 800 and 960
      %

      tof.mex('tof_camera_config_set_mlx_mipi_speed', obj.mPtr, mipi_speed);
    end

    function ret_val = getMipiSpeed(obj)
      % getMipiSpeed
      %   Get the mipi speed
      %
      % Output:
      %   ret_val - numeric - The mipi speed
      %

      ret_val = tof.mex('tof_camera_config_get_mipi_speed', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setMipiSpeed(obj, mipi_speed)
      % setMipiSpeed
      %   Set the mipi speed
      %
      % Inputs:
      %   mipi_speed - numeric - The mipi speed
      %

      tof.mex('tof_camera_config_set_mipi_speed', obj.mPtr, mipi_speed);
    end

    function ret_val = getMlxPreheat(obj, frame)
      % getMlxPreheat
      %   Get the preheat time
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - The preheat time
      %

      ret_val = tof.mex('tof_camera_config_get_mlx_preheat', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setMlxPreheat(obj, frame, preheat)
      % setMlxPreheat
      %   Set the preheat time
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   preheat - numeric - The preheat time
      %

      tof.mex('tof_camera_config_set_mlx_preheat', obj.mPtr, frame, preheat);
    end

    function ret_val = getMlxPreheatEnable(obj, frame)
      % getMlxPreheatEnable
      %   Get enable preheat
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - logical - Enable preheating
      %

      ret_val = tof.mex('tof_camera_config_get_mlx_preheat_enable', obj.mPtr, frame);
    end

    function setMlxPreheatEnable(obj, frame, enable)
      % setMlxPreheatEnable
      %   Set enable preheat
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   enable - logical - Enable preheating
      %

      tof.mex('tof_camera_config_set_mlx_preheat_enable', obj.mPtr, frame, enable);
    end

    function setOptKeaNiosHex(obj, file_location)
      % setOptKeaNiosHex
      %   Set the Nios hex for the keaB cameras
      %
      % Inputs:
      %   file_location - string - File location of the nios hex file
      %

      tof.mex('tof_camera_config_set_opt_kea_nios_hex', obj.mPtr, file_location);
    end

    function ret_val = getOptKeaSubQuad(obj)
      % getOptKeaSubQuad
      %   Get the opt8241 number of sub and quad frames
      %
      % Output:
      %   ret_val - array[numeric] - Number of subs and quads respectively
      %

      ret_val = tof.mex('tof_camera_config_get_opt_kea_sub_quad', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setOptKeaSubQuad(obj, subs_quads)
      % setOptKeaSubQuad
      %   Set the opt8241 number of sub and quad frames
      %
      % Inputs:
      %   subs_quads - array[numeric] - Number of subs and quads respectively
      %

      tof.mex('tof_camera_config_set_opt_kea_sub_quad', obj.mPtr, int32(subs_quads));
    end

    function ret_val = getProcessing(obj)
      % getProcessing
      %   Get the processing configuration for when the depth is calculated on
      %   camera
      %
      % Output:
      %   ret_val - ProcessingConfig - The pipeline processing to be done on
      %                                camera
      %

      ret_val = tof.mex('tof_camera_config_get_processing', obj.mPtr);
      ret_val = tof.ProcessingConfig(ret_val);
    end

    function setProcessing(obj, processing)
      % setProcessing
      %   Set the processing configuration for when the depth is calculated on
      %   camera
      %
      % Inputs:
      %   processing - ProcessingConfig - The pipeline processing to be done on
      %                                   camera
      %

      tof.mex('tof_camera_config_set_processing', obj.mPtr, processing.get_object_ptr());
    end

    function ret_val = getFrameTime(obj, frame)
      % getFrameTime
      %   Get the time this depth frame will take. This is useful when you want
      %   a steady but lower framerate.
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - Frame time in micro seconds
      %

      ret_val = tof.mex('tof_camera_config_get_frame_time', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setFrameTime(obj, frame, frame_time)
      % setFrameTime
      %   Set the time this depth frame will take. This is useful when you want
      %   a steady but lower framerate.
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   frame_time - numeric - Frame time in micro seconds
      %

      tof.mex('tof_camera_config_set_frame_time', obj.mPtr, frame, frame_time);
    end

    function ret_val = getGlobalPhaseOffset(obj, frame)
      % getGlobalPhaseOffset
      %   Get phase offset for illumination and sensor. The phase offsets are
      %   two floating point values corresponding to the phase delay for the
      %   illumination and sensor respectively. The values should be between 0.0
      %   and 1.0 where 0.0 is no phase offset and 1.0 is 360 degree phase
      %   offset
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - array[numeric] - Image is mirrored
      %

      ret_val = tof.mex('tof_camera_config_get_global_phase_offset', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setGlobalPhaseOffset(obj, frame, phase_offset)
      % setGlobalPhaseOffset
      %   Set phase offset for illumination and sensor. The phase offsets are
      %   two floating point values corresponding to the phase delay for the
      %   illumination and sensor respectively. The values should be between 0.0
      %   and 1.0 where 0.0 is no phase offset and 1.0 is 360 degree phase
      %   offset
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   phase_offset - array[numeric] - Image is mirrored
      %

      tof.mex('tof_camera_config_set_global_phase_offset', obj.mPtr, frame, single(phase_offset));
    end

    function ret_val = getIlluminationMode(obj, frame)
      % getIlluminationMode
      %   Get the illumination mode
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - numeric - Illumination mode. 0 is actively modulating, 2 is
      %                       active low (illumination disabled) and 3 is active
      %                       high
      %

      ret_val = tof.mex('tof_camera_config_get_illumination_mode', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setIlluminationMode(obj, frame, illumination_mode)
      % setIlluminationMode
      %   Set the illumination mode
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   illumination_mode - numeric - Illumination mode. 0 is actively
      %                                 modulating, 2 is active low
      %                                 (illumination disabled) and 3 is active
      %                                 high
      %

      tof.mex('tof_camera_config_set_illumination_mode', obj.mPtr, frame, illumination_mode);
    end

    function ret_val = getImageOffsets(obj, frame)
      % getImageOffsets
      %   Get the image offsets. This configuration is only for sensors with
      %   column phase offsets.
      %
      % Inputs:
      %   frame - numeric - Frame number
      %
      % Output:
      %   ret_val - vector[numeric] - The image offset per column in 0.0 to 1.0
      %

      ret_val = tof.mex('tof_camera_config_get_image_offsets', obj.mPtr, frame);
      ret_val = double(ret_val);
    end

    function setImageOffsets(obj, frame, image_offsets)
      % setImageOffsets
      %   Set the image offsets. This configuration is only for sensors with
      %   column phase offsets.
      %
      % Inputs:
      %   frame - numeric - Frame number
      %   image_offsets - vector[numeric] - The image offset per column in 0.0
      %                                     to 1.0
      %

      tof.mex('tof_camera_config_set_image_offsets', obj.mPtr, frame, single(image_offsets));
    end

    function ret_val = getRgbWidth(obj)
      % getRgbWidth
      %   Get the width of the rgb sensor
      %
      % Output:
      %   ret_val - numeric - Width in pixels
      %

      ret_val = tof.mex('tof_camera_config_get_rgb_width', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getRgbHeight(obj)
      % getRgbHeight
      %   Get the height of the rgb sensor
      %
      % Output:
      %   ret_val - numeric - Height in pixels
      %

      ret_val = tof.mex('tof_camera_config_get_rgb_height', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getTriggerMode(obj)
      % getTriggerMode
      %   Get the trigger mode
      %
      % Output:
      %   ret_val - TriggerMode - The trigger mode of the camera
      %

      ret_val = tof.mex('tof_camera_config_get_trigger_mode', obj.mPtr);
    end

    function setTriggerMode(obj, trigger_mode)
      % setTriggerMode
      %   Set the trigger mode
      %
      % Inputs:
      %   trigger_mode - TriggerMode - The trigger mode of the camera
      %

      tof.mex('tof_camera_config_set_trigger_mode', obj.mPtr, uint32(trigger_mode));
    end

    function ret_val = getDepthFps(obj)
      % getDepthFps
      %   Get the maximum theoretical fps the camera can run
      %
      % Output:
      %   ret_val - numeric - The fps in Hz
      %

      ret_val = tof.mex('tof_camera_config_get_depth_fps', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getLightPower(obj)
      % getLightPower
      %   Get the light power
      %
      % Output:
      %   ret_val - LightPower - Light power
      %

      ret_val = tof.mex('tof_camera_config_get_light_power', obj.mPtr);
    end

    function setLightPower(obj, light_power)
      % setLightPower
      %   Set the light power
      %
      % Inputs:
      %   light_power - LightPower - Light power
      %

      tof.mex('tof_camera_config_set_light_power', obj.mPtr, uint32(light_power));
    end

    function ret_val = hasLightPower(obj)
      % hasLightPower
      %   Light power is set
      %
      % Output:
      %   ret_val - logical - Has light power
      %

      ret_val = tof.mex('tof_camera_config_has_light_power', obj.mPtr);
    end

  end
end
