import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .processing_config import *


class CameraType:
    """
    ToF cameras made by Chronoptics
    """

    OPT8241 = 1
    "Original Moa and first generation Kea camera with opt sensor"

    OPT8241_EXT = 4
    "First generation Kea camera with opt sensor and external modulation"

    MLX75027 = 6
    "Newer generation Kea cameras with mlx sensor"

    VD55H1 = 9
    "Kea st camera"



class SensorMode:
    """
    Operating mode of the cameras' sensor
    """

    NORMAL = 0
    "Normal mode ToF sensor mod (A - B)"

    COMMON = 1
    "Common mode (A + B)"

    EXTERNAL_DEMOD = 4
    "Use external demodulation"

    EXTERNAL_SIGNAL = 5
    "Use the light source as an input"

    TAP_A = 8
    "Only tap a is read out"

    TAP_B = 9
    "Only tap b is read out"

    TAP_AB = 10
    "Both A and B tap are read out, this means double the data transmission"

    TAP_AB_HDR = 12
    "Both A and B tap are read out for both a short and long integration time"



class TriggerMode:
    """
    The trigger mode of the camera
    """

    CONTINUOUS = 0
    "The camera is continuously running and outputting frames"

    HARDWARE = 1
    "The camera waits for a hardware trigger"

    SOFTWARE = 2
    "The camera waits for a software trigger"



class LightPower:
    """
    The amount of light power to output
    """

    FULL = 0
    "Full, 100% light power, all 4 vcsels operating"

    HALF = 1
    "Half, 50% light power, 2 vcsels operating"

    ZERO = 2
    "Zero, 0% light power, 0 vcsels operating"



class Roi():
    """
    Region of interest
    """
    def __del__(self):
        if self._ptr:
            lib.tof_roi_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new roi class
        ----------

        Default construct roi
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def sensorRows(self) -> int:
        """
        Get the number of rows in the image sensor

        Returns
        ----------
        int
            Number of rows
        """
        return lib.tof_roi_sensor_rows(self._ptr, get_error())


    def sensorCols(self) -> int:
        """
        Get the number of columns in the image sensor

        Returns
        ----------
        int
            Number of columsn
        """
        return lib.tof_roi_sensor_cols(self._ptr, get_error())


    def getRowOffset(self) -> int:
        """
        Get the row offset in pixels

        Returns
        ----------
        int
            Row offset
        """
        return lib.tof_roi_get_row_offset(self._ptr, get_error())


    def setRowOffset(self, row_offset: int) -> None:
        """
        Set the row offset in pixels

        Parameters
        ----------
        row_offset : int
            Row offset
        """
        return lib.tof_roi_set_row_offset(self._ptr, row_offset, get_error())


    def getColOffset(self) -> int:
        """
        Get the column offset in pixels

        Returns
        ----------
        int
            Column offset
        """
        return lib.tof_roi_get_col_offset(self._ptr, get_error())


    def setColOffset(self, col_offset: int) -> None:
        """
        Set the column offset in pixels

        Parameters
        ----------
        col_offset : int
            Column offset
        """
        return lib.tof_roi_set_col_offset(self._ptr, col_offset, get_error())


    def getImgRows(self) -> int:
        """
        Get the number of rows in the output image

        Returns
        ----------
        int
            Image rows
        """
        return lib.tof_roi_get_img_rows(self._ptr, get_error())


    def setImgRows(self, img_rows: int) -> None:
        """
        Set the number of rows in the output image

        Parameters
        ----------
        img_rows : int
            Image rows
        """
        return lib.tof_roi_set_img_rows(self._ptr, img_rows, get_error())


    def getImgCols(self) -> int:
        """
        Get the number of columns in the output image

        Returns
        ----------
        int
            Image columns
        """
        return lib.tof_roi_get_img_cols(self._ptr, get_error())


    def setImgCols(self, img_cols: int) -> None:
        """
        Set the number of columns in the output image

        Parameters
        ----------
        img_cols : int
            Image columns
        """
        return lib.tof_roi_set_img_cols(self._ptr, img_cols, get_error())


class CameraConfig():
    """
    This class allows you to view/edit the camera settings
    """
    def __del__(self):
        if self._ptr:
            lib.tof_camera_config_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, file_location: Optional[str] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new camera_config class
        ----------

        Read camera config from disk

        Parameters
        ----------
        file_location : str
                Location of camera config on disk
        ----------

        Create camera config from camera type

        Parameters
        ----------
        camera_type : CameraType
                Camera type
        ----------

        Create camera config from the four types

        Parameters
        ----------
        depth_sensor_type : DepthSensorType
                Depth sensor type
        sensor_board_type : SensorBoardType
                Sensor board type
        light_source_type : LightSourceType
                Light source type
        rgb_sensor_type : RgbSensorType
                RGB sensor type
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif file_location != None:
            self._ptr = lib.tof_camera_config_new_from_disk(file_location.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def write(self, file_location: str) -> None:
        """
        Write camera config to disk

        Parameters
        ----------
        file_location : str
            Location to save camera config to
        """
        return lib.tof_camera_config_write(self._ptr, file_location.encode("utf-8"), get_error())


    def getType(self) -> CameraType:
        """
        Get the camera type

        Returns
        ----------
        CameraType
            Camera type
        """
        return lib.tof_camera_config_get_type(self._ptr, get_error())


    def reset(self) -> None:
        """
        Reset camera config to default
        """
        return lib.tof_camera_config_reset(self._ptr, get_error())


    def frameSize(self) -> int:
        """
        Get the amount of frames in the config. Having multiple frames allows you to
        configure different modulation frequencies or integration times per frame.

        Returns
        ----------
        int
            Frame size
        """
        return lib.tof_camera_config_frame_size(self._ptr, get_error())


    def addFrame(self) -> None:
        """
        Add a default frame to the camera config
        """
        return lib.tof_camera_config_add_frame(self._ptr, get_error())


    def defaultProcessing(self) -> ProcessingConfig:
        """
        Generate default processing config from camera configuration

        Returns
        ----------
        ProcessingConfig
            The default processing config
        """
        return ProcessingConfig(ptr = lib.tof_camera_config_default_processing(self._ptr, get_error()))


    def eraseFrame(self, frame: int) -> None:
        """
        Erase specified frame

        Parameters
        ----------
        frame : int
            Frame number
        """
        return lib.tof_camera_config_erase_frame(self._ptr, frame, get_error())


    def getPhaseShifts(self, frame: int) -> List[float]:
        """
        Get the phase shifts. Value ranges from 0.0 to 1.0 which translates to 0-2
        pi

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        List[float]
            Phase shifts
        """
        size = lib.tof_camera_config_get_phase_shifts(self._ptr, frame, ctypes.c_float(), 0, get_error())
        array = (ctypes.c_float * size)()
        lib.tof_camera_config_get_phase_shifts(self._ptr, frame, array, size, get_error())
        return array[:]


    def setPhaseShifts(self, frame: int, phase_shifts: List[float]) -> None:
        """
        Set the phase shifts. Value ranges from 0.0 to 1.0 which translates to 0-2
        pi

        Parameters
        ----------
        frame : int
            Frame number
        phase_shifts : List[float]
            Phase shifts
        """
        return lib.tof_camera_config_set_phase_shifts(self._ptr, frame, (ctypes.c_float * len(phase_shifts))(*phase_shifts), len(phase_shifts), get_error())


    def getIntegrationTime(self, frame: int) -> List[int]:
        """
        Get the integration time

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        List[int]
            Integration time in micro seconds
        """
        size = lib.tof_camera_config_get_integration_time(self._ptr, frame, ctypes.c_uint32(), 0, get_error())
        array = (ctypes.c_uint32 * size)()
        lib.tof_camera_config_get_integration_time(self._ptr, frame, array, size, get_error())
        return array[:]


    def setIntegrationTime(self, frame: int, integration_time: List[int]) -> None:
        """
        Set the integration time

        Parameters
        ----------
        frame : int
            Frame number
        integration_time : List[int]
            Integration time in micro seconds
        """
        return lib.tof_camera_config_set_integration_time(self._ptr, frame, (ctypes.c_uint32 * len(integration_time))(*integration_time), len(integration_time), get_error())


    def getSensorMode(self, frame: int) -> SensorMode:
        """
        Get the mode to set the camera's depth sensor to

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        SensorMode
            Sensor mode
        """
        return lib.tof_camera_config_get_sensor_mode(self._ptr, frame, get_error())


    def setSensorMode(self, frame: int, sensor_mode: SensorMode) -> None:
        """
        Set the mode to set the camera's depth sensor to

        Parameters
        ----------
        frame : int
            Frame number
        sensor_mode : SensorMode
            Sensor mode
        """
        return lib.tof_camera_config_set_sensor_mode(self._ptr, frame, sensor_mode, get_error())


    def getModulationFrequency(self, frame: int) -> float:
        """
        Get the modulation frequency

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        float
            Modulation frequency in MHz
        """
        return lib.tof_camera_config_get_modulation_frequency(self._ptr, frame, get_error())


    def setModulationFrequency(self, frame: int, modulation_frequency: float) -> None:
        """
        Set the modulation frequency

        Parameters
        ----------
        frame : int
            Frame number
        modulation_frequency : float
            Modulation frequency in MHz
        """
        return lib.tof_camera_config_set_modulation_frequency(self._ptr, frame, modulation_frequency, get_error())


    def getDutyCycle(self, frame: int) -> float:
        """
        Get the duty cycle

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        float
            Duty cycle 0.0 to 1.0
        """
        return lib.tof_camera_config_get_duty_cycle(self._ptr, frame, get_error())


    def setDutyCycle(self, frame: int, duty_cycle: float) -> None:
        """
        Set the duty cycle

        Parameters
        ----------
        frame : int
            Frame number
        duty_cycle : float
            Duty cycle 0.0 to 1.0
        """
        return lib.tof_camera_config_set_duty_cycle(self._ptr, frame, duty_cycle, get_error())


    def getRoi(self, frame: int) -> Roi:
        """
        Get the region of interest

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        Roi
            Region of interest
        """
        return Roi(ptr = lib.tof_camera_config_get_roi(self._ptr, frame, get_error()))


    def setRoi(self, frame: int, roi: Roi) -> None:
        """
        Set the region of interest

        Parameters
        ----------
        frame : int
            Frame number
        roi : Roi
            Region of interest
        """
        return lib.tof_camera_config_set_roi(self._ptr, frame, roi._ptr, get_error())


    def getBinning(self, frame: int) -> int:
        """
        Get the amount of binning

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        int
            Amount of binning 0=none, 1=x2, 2=x4, 3=x8
        """
        return lib.tof_camera_config_get_binning(self._ptr, frame, get_error())


    def setBinning(self, frame: int, binning: int) -> None:
        """
        Set the amount of binning

        Parameters
        ----------
        frame : int
            Frame number
        binning : int
            Amount of binning 0=none, 1=x2, 2=x4, 3=x8
        """
        return lib.tof_camera_config_set_binning(self._ptr, frame, binning, get_error())


    def getFlip(self, frame: int) -> bool:
        """
        Get is the image flipped

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        bool
            Image is flipped
        """
        return lib.tof_camera_config_get_flip(self._ptr, frame, get_error())


    def setFlip(self, frame: int, flip: bool) -> None:
        """
        Set is the image flipped

        Parameters
        ----------
        frame : int
            Frame number
        flip : bool
            Image is flipped
        """
        return lib.tof_camera_config_set_flip(self._ptr, frame, flip, get_error())


    def getMirror(self, frame: int) -> bool:
        """
        Get is the image mirrored

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        bool
            Image is mirrored
        """
        return lib.tof_camera_config_get_mirror(self._ptr, frame, get_error())


    def setMirror(self, frame: int, mirror: bool) -> None:
        """
        Set is the image mirrored

        Parameters
        ----------
        frame : int
            Frame number
        mirror : bool
            Image is mirrored
        """
        return lib.tof_camera_config_set_mirror(self._ptr, frame, mirror, get_error())


    def getGain(self) -> float:
        """
        Get the image sensor gain

        Returns
        ----------
        float
            Image sensor gain
        """
        return lib.tof_camera_config_get_gain(self._ptr, get_error())


    def setGain(self, gain: float) -> None:
        """
        Set the image sensor gain

        Parameters
        ----------
        gain : float
            Image sensor gain
        """
        return lib.tof_camera_config_set_gain(self._ptr, gain, get_error())


    def getSyncMode(self) -> int:
        """
        Get the kind of camera synchronization

        Returns
        ----------
        int
            The kind of sync mode. 0 waits for the vsync pulse before starting the
            next frame. 1 optional vsync 2 is no frame syncing
        """
        return lib.tof_camera_config_get_sync_mode(self._ptr, get_error())


    def setSyncMode(self, sync_mode: int) -> None:
        """
        Set the kind of camera synchronization

        Parameters
        ----------
        sync_mode : int
            The kind of sync mode. 0 waits for the vsync pulse before starting the
            next frame. 1 optional vsync 2 is no frame syncing
        """
        return lib.tof_camera_config_set_sync_mode(self._ptr, sync_mode, get_error())


    def maximumIntegrationTime(self, frame: int) -> List[int]:
        """
        Get the maximum integration time possible

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        List[int]
            Maximum integration time in micro seconds
        """
        size = lib.tof_camera_config_maximum_integration_time(self._ptr, frame, ctypes.c_uint32(), 0, get_error())
        array = (ctypes.c_uint32 * size)()
        lib.tof_camera_config_maximum_integration_time(self._ptr, frame, array, size, get_error())
        return array[:]


    def getDac(self) -> List[int]:
        """
        Get the DAC gain

        Returns
        ----------
        List[int]
            The DAC gain
        """
        size = lib.tof_camera_config_get_dac(self._ptr, ctypes.c_uint16(), 0, get_error())
        array = (ctypes.c_uint16 * size)()
        lib.tof_camera_config_get_dac(self._ptr, array, size, get_error())
        return array[:]


    def setDac(self, dac: List[int]) -> None:
        """
        Set the DAC gain

        Parameters
        ----------
        dac : List[int]
            The DAC gain
        """
        return lib.tof_camera_config_set_dac(self._ptr, (ctypes.c_uint16 * len(dac))(*dac), len(dac), get_error())


    def getMlxMipiSpeed(self) -> int:
        """
        Get the mipi speed for the MLX75027

        Returns
        ----------
        int
            The mipi speed. Can only be the values 300, 600, 704, 800 and 960
        """
        return lib.tof_camera_config_get_mlx_mipi_speed(self._ptr, get_error())


    def setMlxMipiSpeed(self, mipi_speed: int) -> None:
        """
        Set the mipi speed for the MLX75027

        Parameters
        ----------
        mipi_speed : int
            The mipi speed. Can only be the values 300, 600, 704, 800 and 960
        """
        return lib.tof_camera_config_set_mlx_mipi_speed(self._ptr, mipi_speed, get_error())


    def getMipiSpeed(self) -> int:
        """
        Get the mipi speed

        Returns
        ----------
        int
            The mipi speed
        """
        return lib.tof_camera_config_get_mipi_speed(self._ptr, get_error())


    def setMipiSpeed(self, mipi_speed: int) -> None:
        """
        Set the mipi speed

        Parameters
        ----------
        mipi_speed : int
            The mipi speed
        """
        return lib.tof_camera_config_set_mipi_speed(self._ptr, mipi_speed, get_error())


    def getMlxPreheat(self, frame: int) -> int:
        """
        Get the preheat time

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        int
            The preheat time
        """
        return lib.tof_camera_config_get_mlx_preheat(self._ptr, frame, get_error())


    def setMlxPreheat(self, frame: int, preheat: int) -> None:
        """
        Set the preheat time

        Parameters
        ----------
        frame : int
            Frame number
        preheat : int
            The preheat time
        """
        return lib.tof_camera_config_set_mlx_preheat(self._ptr, frame, preheat, get_error())


    def getMlxPreheatEnable(self, frame: int) -> bool:
        """
        Get enable preheat

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        bool
            Enable preheating
        """
        return lib.tof_camera_config_get_mlx_preheat_enable(self._ptr, frame, get_error())


    def setMlxPreheatEnable(self, frame: int, enable: bool) -> None:
        """
        Set enable preheat

        Parameters
        ----------
        frame : int
            Frame number
        enable : bool
            Enable preheating
        """
        return lib.tof_camera_config_set_mlx_preheat_enable(self._ptr, frame, enable, get_error())


    def setOptKeaNiosHex(self, file_location: str) -> None:
        """
        Set the Nios hex for the keaB cameras

        Parameters
        ----------
        file_location : str
            File location of the nios hex file
        """
        return lib.tof_camera_config_set_opt_kea_nios_hex(self._ptr, file_location.encode("utf-8"), get_error())


    def getOptKeaSubQuad(self) -> List[int]:
        """
        Get the opt8241 number of sub and quad frames

        Returns
        ----------
        List[int]
            Number of subs and quads respectively
        """
        array = lib.tof_camera_config_get_opt_kea_sub_quad(self._ptr, get_error())
        return array.contents[:]


    def setOptKeaSubQuad(self, subs_quads: List[int]) -> None:
        """
        Set the opt8241 number of sub and quad frames

        Parameters
        ----------
        subs_quads : List[int]
            Number of subs and quads respectively
        """
        return lib.tof_camera_config_set_opt_kea_sub_quad(self._ptr, (ctypes.c_int32 * 2)(*subs_quads), get_error())


    def getProcessing(self) -> ProcessingConfig:
        """
        Get the processing configuration for when the depth is calculated on camera

        Returns
        ----------
        ProcessingConfig
            The pipeline processing to be done on camera
        """
        return ProcessingConfig(ptr = lib.tof_camera_config_get_processing(self._ptr, get_error()))


    def setProcessing(self, processing: ProcessingConfig) -> None:
        """
        Set the processing configuration for when the depth is calculated on camera

        Parameters
        ----------
        processing : ProcessingConfig
            The pipeline processing to be done on camera
        """
        return lib.tof_camera_config_set_processing(self._ptr, processing._ptr, get_error())


    def getFrameTime(self, frame: int) -> int:
        """
        Get the time this depth frame will take. This is useful when you want a
        steady but lower framerate.

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        int
            Frame time in micro seconds
        """
        return lib.tof_camera_config_get_frame_time(self._ptr, frame, get_error())


    def setFrameTime(self, frame: int, frame_time: int) -> None:
        """
        Set the time this depth frame will take. This is useful when you want a
        steady but lower framerate.

        Parameters
        ----------
        frame : int
            Frame number
        frame_time : int
            Frame time in micro seconds
        """
        return lib.tof_camera_config_set_frame_time(self._ptr, frame, frame_time, get_error())


    def getGlobalPhaseOffset(self, frame: int) -> List[float]:
        """
        Get phase offset for illumination and sensor. The phase offsets are two
        floating point values corresponding to the phase delay for the illumination
        and sensor respectively. The values should be between 0.0 and 1.0 where 0.0
        is no phase offset and 1.0 is 360 degree phase offset

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        List[float]
            Image is mirrored
        """
        array = lib.tof_camera_config_get_global_phase_offset(self._ptr, frame, get_error())
        return array.contents[:]


    def setGlobalPhaseOffset(self, frame: int, phase_offset: List[float]) -> None:
        """
        Set phase offset for illumination and sensor. The phase offsets are two
        floating point values corresponding to the phase delay for the illumination
        and sensor respectively. The values should be between 0.0 and 1.0 where 0.0
        is no phase offset and 1.0 is 360 degree phase offset

        Parameters
        ----------
        frame : int
            Frame number
        phase_offset : List[float]
            Image is mirrored
        """
        return lib.tof_camera_config_set_global_phase_offset(self._ptr, frame, (ctypes.c_float * 2)(*phase_offset), get_error())


    def getIlluminationMode(self, frame: int) -> int:
        """
        Get the illumination mode

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        int
            Illumination mode. 0 is actively modulating, 2 is active low
            (illumination disabled) and 3 is active high
        """
        return lib.tof_camera_config_get_illumination_mode(self._ptr, frame, get_error())


    def setIlluminationMode(self, frame: int, illumination_mode: int) -> None:
        """
        Set the illumination mode

        Parameters
        ----------
        frame : int
            Frame number
        illumination_mode : int
            Illumination mode. 0 is actively modulating, 2 is active low
            (illumination disabled) and 3 is active high
        """
        return lib.tof_camera_config_set_illumination_mode(self._ptr, frame, illumination_mode, get_error())


    def getImageOffsets(self, frame: int) -> List[float]:
        """
        Get the image offsets. This configuration is only for sensors with column
        phase offsets.

        Parameters
        ----------
        frame : int
            Frame number

        Returns
        ----------
        List[float]
            The image offset per column in 0.0 to 1.0
        """
        size = lib.tof_camera_config_get_image_offsets(self._ptr, frame, ctypes.c_float(), 0, get_error())
        array = (ctypes.c_float * size)()
        lib.tof_camera_config_get_image_offsets(self._ptr, frame, array, size, get_error())
        return array[:]


    def setImageOffsets(self, frame: int, image_offsets: List[float]) -> None:
        """
        Set the image offsets. This configuration is only for sensors with column
        phase offsets.

        Parameters
        ----------
        frame : int
            Frame number
        image_offsets : List[float]
            The image offset per column in 0.0 to 1.0
        """
        return lib.tof_camera_config_set_image_offsets(self._ptr, frame, (ctypes.c_float * len(image_offsets))(*image_offsets), len(image_offsets), get_error())


    def getRgbWidth(self) -> int:
        """
        Get the width of the rgb sensor

        Returns
        ----------
        int
            Width in pixels
        """
        return lib.tof_camera_config_get_rgb_width(self._ptr, get_error())


    def getRgbHeight(self) -> int:
        """
        Get the height of the rgb sensor

        Returns
        ----------
        int
            Height in pixels
        """
        return lib.tof_camera_config_get_rgb_height(self._ptr, get_error())


    def getTriggerMode(self) -> TriggerMode:
        """
        Get the trigger mode

        Returns
        ----------
        TriggerMode
            The trigger mode of the camera
        """
        return lib.tof_camera_config_get_trigger_mode(self._ptr, get_error())


    def setTriggerMode(self, trigger_mode: TriggerMode) -> None:
        """
        Set the trigger mode

        Parameters
        ----------
        trigger_mode : TriggerMode
            The trigger mode of the camera
        """
        return lib.tof_camera_config_set_trigger_mode(self._ptr, trigger_mode, get_error())


    def getDepthFps(self) -> float:
        """
        Get the maximum theoretical fps the camera can run

        Returns
        ----------
        float
            The fps in Hz
        """
        return lib.tof_camera_config_get_depth_fps(self._ptr, get_error())


    def getLightPower(self) -> LightPower:
        """
        Get the light power

        Returns
        ----------
        LightPower
            Light power
        """
        return lib.tof_camera_config_get_light_power(self._ptr, get_error())


    def setLightPower(self, light_power: LightPower) -> None:
        """
        Set the light power

        Parameters
        ----------
        light_power : LightPower
            Light power
        """
        return lib.tof_camera_config_set_light_power(self._ptr, light_power, get_error())


    def hasLightPower(self) -> bool:
        """
        Light power is set

        Returns
        ----------
        bool
            Has light power
        """
        return lib.tof_camera_config_has_light_power(self._ptr, get_error())


def cameraConfigEqual(lhs: CameraConfig, rhs: CameraConfig) -> bool:
    """
    Test whether camera configurations are equal

    Parameters
    ----------
    lhs : CameraConfig
        First camera config
    rhs : CameraConfig
        Second camera config

    Returns
    ----------
    bool
        Camera configurations are equal
    """
    return lib.tof_camera_config_equal(lhs._ptr, rhs._ptr, get_error())


def lightPowerName(light_power: LightPower) -> str:
    """
    Get the name of the light power

    Parameters
    ----------
    light_power : LightPower
        Light Power

    Returns
    ----------
    str
        Name
    """
    return lib.tof_light_power_name(light_power, get_error()).decode("utf-8")


